/**
 * main file of project
 *
 * @author		Sven Skrabal
 * @copyright	(C) 2007 Sven Skrabal
 * @license		GNU General Public License (v3)
 *
 */

#include "stdafx.h"

/**
 * thread function that receives first bytes of client request to check for SOCKS4 protocol
 * depending on this data and the request also SOCKS5 could be implemented easily
 *
 * @param	sClientSocket	pointer to a socket from new connected client
 * @return	returns -1 on error, a value greater or equal to 0 on success
 *
 */
DWORD WINAPI detectProxyType(SOCKET *sClientSocket)
{
	int iRetVal;
	char szBuf[2048];
	struct timeval tvTimeout;
	FD_SET fdSet;

	tvTimeout.tv_sec = 0;
	tvTimeout.tv_usec = REQUEST_TIMEOUT;

	//wait until client has sended SOCKS protocol data
	if(select(*(sClientSocket), &fdSet, NULL, NULL, &tvTimeout)!=0)
	{
		//receive client request
		iRetVal = recv(*(sClientSocket), szBuf, 2048, 0);

		//request is only complete if buffer contains more than 8 bytes
		if(iRetVal > 8)
		{
			//check client data for SOCKS4
			if(szBuf[0] == 4 && (szBuf[1] == 1 || szBuf[1] == 2))
			{
				//SOCKS4 connection detected - handle that in function
				startSocks4(sClientSocket, szBuf, iRetVal);

				closesocket(*(sClientSocket));
				*sClientSocket = INVALID_SOCKET;

				return 0;
			}
		}
	}

	//properly close socket and set pointer to invalid
	closesocket(*(sClientSocket));
	*sClientSocket = INVALID_SOCKET;

	return -1;
}

/**
 * main function of whole application
 *
 */
int _tmain(int argc, _TCHAR* argv[])
{
	SOCKET sAcceptSocket, sClientSocket[MAX_CONNECTIONS];
	SOCKADDR_IN siAddr;
	DWORD dwThrID;
	WSADATA WsaData;

	//init winsock
	WSAStartup(MAKEWORD(2, 0), &WsaData);

	//open listening socket for incoming connections
	createListenSocket(&sAcceptSocket, &siAddr, LISTEN_PORT, 0, 1);

	//initialize array
	for(int i=0; i<MAX_CONNECTIONS; i++)
	{
		sClientSocket[i] = INVALID_SOCKET;
	}

	while(true)
	{
		for(int i=0; i<MAX_CONNECTIONS; i++)
		{
			//iterate over all entries to find an unused entry
			if(sClientSocket[i] == INVALID_SOCKET)
			{
				//accept the new client (blocking call)
				sClientSocket[i] = accept(sAcceptSocket, NULL, NULL);

				//check for successfully connected client
				if(sClientSocket[i] != INVALID_SOCKET)
				{
					//start thread for proxy detection
					CreateThread(0, 0, (LPTHREAD_START_ROUTINE)&detectProxyType, &sClientSocket[i], 0, &dwThrID);
				}
				else
				{
					//on error make socket available again
					sClientSocket[i] = INVALID_SOCKET;
				}

				break;
			}
		}
	}

	return 0;
}
