/**
 * SOCKS4 related functions
 *
 * @author		Sven Skrabal
 * @copyright	(C) 2007 Sven Skrabal
 * @license		GNU General Public License (v3)
 *
 */

#include "stdafx.h"

/**
 * reads the SOCKS4 request protocol data and performs its actions
 * supported is connecting to a remote server and listening on a new socket
 *
 * @param	sClientSocket	valid socket from client that has connected
 * @param	szBuf			previously read data from client for proxy request
 * @param	iReceived		how many bytes are given in szBuf
 * @return	returns -1 on error, value greater or equal on success
 *
 */
int startSocks4(SOCKET *sClientSocket, char *szBuf, int iReceived)
{
	SOCKET sServer, sServer2;
	unsigned int iPort;
	char szIp[16], szTemp[10];
	SOCKADDR_IN siAddr;
	FD_SET fdSet;
	struct timeval tvTimeout;

	tvTimeout.tv_sec = 0;
	tvTimeout.tv_usec = ACCEPT_TIMEOUT;

	//expand ip and port
	iPort = (szBuf[2]*256) + szBuf[3];
	sprintf(szIp, "%i.%i.%i.%i", (BYTE)szBuf[4], (BYTE)szBuf[5], (BYTE)szBuf[6], (BYTE)szBuf[7]);

	//check for connect command
	if(szBuf[1] == 1)
	{
		printf("CONNECT : %s:%i\n", szIp, iPort);

		//connect to requested destination and redirect data
		if(createSocketAndConnect(&sServer, &siAddr, szIp, iPort, 0)==0)
		{
			//send success packet
			szTemp[0] = 0;
			szTemp[1] = 90;
			memcpy(szTemp+2, szBuf+2, 6);

			send(*(sClientSocket), szTemp, 8, 0);

			//now redirect the two connections
			redirectSockets(sClientSocket, &sServer);

			printf("CLOSE   : %s:%i\n", szIp, iPort);
			closesocket(sServer);

			return 0;
		}

		printf("CLOSE   : %s:%i\n", szIp, iPort);
	}

	//check for bind command
	if(szBuf[1] == 2)
	{
		printf("BIND    : %s:%i\n", szIp, iPort);

		//create new listen socket and wait for new connections
		if(createListenSocket(&sServer, &siAddr, iPort, 0, 0)==0)
		{
			//send success packet
			szTemp[0] = 0;
			szTemp[1] = 90;
			memcpy(szTemp+2, szBuf+2, 2);
			memset(szTemp+4, 0, 4);

			send(*(sClientSocket), szTemp, 8, 0);

			//initialize structures
			FD_ZERO(&fdSet);
			FD_SET(sServer, &fdSet);

			//check sockets for data
			if(select(0, &fdSet, NULL, NULL, &tvTimeout) != 0 )
			{
				//accept new connection
				sServer2 = accept(sServer, NULL, NULL);

				//if connection was successful - redirect both clients
				if(sServer2 != INVALID_SOCKET)
				{
					//send success package
					send(*(sClientSocket), szTemp, 8, 0);
					
					//now redirect the two connections
					redirectSockets(sClientSocket, &sServer2);

					printf("CLOSE   : %s:%i\n", szIp, iPort);
					closesocket(sServer);
					closesocket(sServer2);

					return 0;
				}
			}
		}

		printf("CLOSE   : %s:%i\n", szIp, iPort);
	}

	//send failure packet on error
	szTemp[0] = 0;
	szTemp[1] = 91;
	memcpy(szTemp+2, szBuf+2, 6);

	send(*(sClientSocket), szTemp, 8, 0);

	//properly close handle and return with error code
	closesocket(sServer);
	return -1;
}
